#!/bin/bash

#no access for group or others to files created as part of the backup.
umask 077
wd=`pwd`

#Backup settings section:
#####################################################################
#Edit variables below:
#Name for computer. This name is used when creating the backup
#filenames.
COMPUTER=my_computer

#Path to file containing a list of directories and files to
#backup. This file is used by tar via the --files-from option.
INCLUDE=/path/to/include.conf

#Path to file containing a list of directories and files to exclude
#from the backup. This file is used by tar via the --exclude-from
#option.
EXCLUDE=/path/to/exclude.conf

#Directory to contain backups.
BACKUPDIR=/backup

#Size of split files.
SPLITSIZE=4000m

#Path to sendmail.
SENDMAIL=/usr/sbin/sendmail

#Person to receive email reports.
MAILTO='someaddress@someplace.com'

#Path to md5sum.
MD5SUM=/usr/bin/md5sum

#Path to tar.
TAR=/bin/tar

#Name and location of split. Split is used to divide the file into
#smaller fragments that can be written to CD or DVD.
SPLIT=/usr/bin/split

#Path to the md5_sums.pl script.
MD5SCRIPT=/path/to/md5_sums.pl

#Path to sort.
SORT=/bin/sort

#####################################################################

#This file will contain the time of the last full backup.
TIMESTAMP=$BACKUPDIR/$COMPUTER-lastfull

#Day of the week e.g. Mon
DOW=`date +%a`
#Date of the Month e.g. 27
DOM=`date +%d`
#Used for timestamp file
NOW=`date +%d-%b`
#Full date e.g. 2006_09_02
FULLDATE=`date +%Y_%m_%d`

#Check for BACKUPDIR
if [ ! -d "$BACKUPDIR" ]; then
    mkdir "$BACKUPDIR"
fi

#Change DOW text to lowercase
DOW=`echo $DOW | tr A-Z a-z`

function check_execution {
    exit_status="$1"
    command="$2"
    if [ $exit_status -ne 0 ]; then
	echo "Backup failed with command $command"
	email "Backup error" "Backup failed with command $command"
	exit "$exit_status"
    fi
}

function email {
    subject="$1"
    message="$2"

echo "From: $COMPUTER
To: $MAILTO
Subject: $subject
Mime-Version: 1.0
Content-Type: text/plain" > /tmp/backup_message$$

    echo "$message" >> /tmp/backup_message$$
    $SENDMAIL -t -oi < /tmp/backup_message$$
    exit_status=$?
    if [ $exit_status -ne 0 ]; then
	echo "Unable to send email using $SENDMAIL"
	exit "$exit_status"
    fi
}

#Create timestamp file if it doesn't exist and do a full backup.
cd "$BACKUPDIR"
if [ ! -f "$TIMESTAMP" ] || [ $DOM = "01" ] ; then

    #Determine md5 sums of files to be backed up
    perl $MD5SCRIPT -i $INCLUDE -e $EXCLUDE -o $COMPUTER-$FULLDATE-MD5_SUMS.txt
    check_execution $? "$MD5SCRIPT -i $INCLUDE -e $EXCLUDE -o $COMPUTER-$FULLDATE-MD5_SUMS.txt"

    #This backup will not be overwritten by this script.
    $TAR -czf $COMPUTER-$FULLDATE.tar.gz --exclude-from $EXCLUDE --files-from $INCLUDE
    check_execution $? "$TAR -czf $COMPUTER-$FULLDATE.tar.gz --exclude-from $EXCLUDE --files-from $INCLUDE"

    #split tar file into smaller parts for CD backup
    $SPLIT -b $SPLITSIZE $COMPUTER-$FULLDATE.tar.gz $COMPUTER-$FULLDATE.tar.gz_part_
    check_execution $? "$SPLIT -b $SPLITSIZE $COMPUTER-$FULLDATE.tar.gz $COMPUTER-$FULLDATE.tar.gz_part_"

    echo "To create original tar file use:
cat $COMPUTER-$FULLDATE.tar.gz_part_* >> $COMPUTER-$FULLDATE.tar.gz
Then use the following to extract as su:
tar xpvf $COMPUTER-$FULLDATE.tar.gz
The md5sum of $COMPUTER-$FULLDATE.tar.gz should be:
" > $COMPUTER-$FULLDATE-README.txt

    #Add md5sum to end of README.txt
    $MD5SUM $COMPUTER-$FULLDATE.tar.gz >> $COMPUTER-$FULLDATE-README.txt
    check_execution $? "$MD5SUM $COMPUTER-$FULLDATE.tar.gz >> $COMPUTER-$FULLDATE-README.txt"

    #Create a list of backup contents sorted by size
    $TAR -tvf $COMPUTER-$FULLDATE.tar.gz | $SORT -k 3 -nr >> $COMPUTER-$FULLDATE-CONTENTS.txt
    check_execution $? "$TAR -tvf $COMPUTER-$FULLDATE.tar.gz | $SORT -k 3 -nr >> $COMPUTER-$FULLDATE-CONTENTS.txt"

    #Remove the full-size file
    rm $COMPUTER-$FULLDATE.tar.gz
    check_execution $? "rm $COMPUTER-$FULLDATE.tar.gz"

    email "Backup successful" "A backup was created for $COMPUTER on $FULLDATE"

    echo $NOW > $TIMESTAMP

    #If it is Sunday, create a full backup. This backup will be
    #overwritten next Sunday.

elif [ $DOW = "sun" ]; then
    #Determine md5 sums of files to be backed up
    perl $MD5SCRIPT -i $INCLUDE -e $EXCLUDE -o $COMPUTER-$FULLDATE-MD5_SUMS.txt
    check_execution $? "$MD5SCRIPT -i $INCLUDE -e $EXCLUDE -o $COMPUTER-$FULLDATE-MD5_SUMS.txt"

    $TAR -czf $COMPUTER-$DOW.tar.gz --exclude-from $EXCLUDE --files-from $INCLUDE
    check_execution $? "$TAR -czf $COMPUTER-$DOW.tar.gz --exclude-from $EXCLUDE --files-from $INCLUDE"

    echo $NOW > $TIMESTAMP
else
    #Write an incremental backup containing files changed
    #since last full backup. This backup will be overwritten
    #the following week.
    NEWER="--newer `cat $TIMESTAMP`"
    $TAR $NEWER -czf $COMPUTER-$DOW.tar.gz --exclude-from $EXCLUDE --files-from $INCLUDE
    check_execution $? "$TAR $NEWER -czf $COMPUTER-$DOW.tar.gz --exclude-from $EXCLUDE --files-from $INCLUDE"
fi

cd "$wd"
exit 0

